/* 

# Copyright (c) 2003 Todd T. Fries <todd@fries.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
# THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

compression
- sha1 based
  - pick a block size, "compress" based on
    sha1 signature
  - perhaps use some kind of info to help generate
    the data again, like number of bits, ??
  - verify that no earlier generated block of data
     generates the data, perhaps keep track of the
     match number
  - make it recognize a pattern the size of the hash,
    make hzip_PATTERN a flag(hash algorithm) for
    bits of repeating natures, implement later
  - header is 
	["hzip" + header size + filesize + pad]
      (allow for signing)
... the point is not to be fast but to
  guarantee shrinkage at the expense of cpu time
  for retrieval
    1024 bytes --> 16 bytes + overhead
*/


#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <openssl/evp.h>

#include "hzip.h"

void
hzip(char *file)
{
	hzip_block_t hb;
	size_t s;
	int i, j;
	u_int8_t buff[CHUNK_ZIP_SIZE];
	int in, out;
	int blocknum = 0;

	hzip_header_t *hh;

	bzero(buff, sizeof(buff));

	in = open(file, O_RDONLY, 600);
	if(in == -1)
		exit(1);
	out= open("/dev/stdout", O_WRONLY, 600);
	if(out == -1)
		exit(1);

	if (debug & D_BASIC) {
		fprintf(stderr, "chunk size: %d  block size: %d\n",
			CHUNK_ZIP_SIZE, sizeof(hzip_block_t));
	}

	while(s = read(in, buff, CHUNK_ZIP_SIZE)) {
		hb.mset = hb.uset = 0xff;

		if ( s != CHUNK_ZIP_SIZE) {
			int rev = revpower(s);
			int pow = TWO_POWER(rev);
			while ( pow < s ) {
				rev++;
				pow = TWO_POWER(rev);
			}
			s = pow;
		}

		for(i = 0; i < s; i++) {
			if (hb.mset)
				hb.mset = mask_set_add(hb.mset, buff[i]);
			if (hb.uset)
				hb.uset = mask_unset_add(hb.uset, buff[i]);
		}
		hb.bits_set = count_bits(buff, s);

   		hash(buff, s, &hb);

		show_hb(&hb);

		// spit hb to stdout
		show_hb(&hb);
		show_hash(buff, blocknum, s);

		write(out, (void *)&hb, sizeof(hzip_block_t));

		blocknum++;
	}
/*
	for( j = 0; j < (sizeof(buff) * 8 * 1000); j++ ) {
		hash( buff, sizeof(buff), &hb );
	}
 */

}
