/* 

# Copyright (c) 2003 Todd T. Fries <todd@fries.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
# THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

struct hzip_header; /* struct written to the file once */
struct hzip_block;  /* struct written to the file many times */
struct hzip_bvars;  /* internal usage struct for passing vars and hzip_blocks */

typedef struct hzip_header hzip_header_t;
typedef struct hzip_block hzip_block_t;
typedef struct hzip_bvars hzip_bvars_t;

struct hzip_header {
	char str[5];
	u_int8_t hlen; // = sizeof(hzip_header_t);
	u_int8_t slen; // stream length
	u_int8_t data[16]; // obviously this can be increased in size
};

struct hzip_block {
	u_int8_t hash[EVP_MAX_MD_SIZE];
	u_int8_t first;
/* thought: force hash type on all blocks so hash type is in the header not
   the block struct?? */
	u_int8_t info;
#define hzip_SHA1	0x01
#define hzip_SHA256	0x02
#define hzip_SHA512	0x03
#define hzip_MD5	0x04

/*  Once things mature, instead of always starting at the beginning, reduce
    the number of computations by 4 again by specifying direction and beginning
    point
#define hzip_TOPDOWN	0x10
#define hzip_MIDUP	0x20
#define hzip_MIDDOWN	0x30
#define hzip_BOTTOMUP	0x40
 */

	u_int8_t len; // size = 2^len
/* len ^ 2 means 0x0f is 2^15 .. big enough for any block, after all we are
                                 talking about arbitrarily breaking a stream
				 of data, not packets, memory, or disk... */

#define hzip_ENDBLOCK	0x10
/* if hzip_ENDBLOCK is part of a len field, then the 1st two bytes of this
   block will be the exact length of this block
   the 'hzip_ENDBLOCK' is only necessary if the end block is not a power
   of two in size
*/
	u_int8_t iter; 	// number of hash collisions - 1 ( 0 = 1st found,
			//				   2 = 2nd found)
	u_int8_t mset;	/* set bits bitmask to speed computation */
	u_int8_t uset; /* unset bits bitmask to speed computation */
	u_int16_t bits_set; /* count number of bits set to speed computation */
};

struct hzip_bvars {
	u_int8_t *buff;
	size_t s;
	hzip_block_t *b;
	u_int8_t max;
	u_int16_t bits;
};


#define TWO_POWER(x)	(1<<x)

//#define CHUNK_ZIP_POWER	10
#define CHUNK_ZIP_POWER	7
#define CHUNK_ZIP_SIZE TWO_POWER(CHUNK_ZIP_POWER)
// always a Power of 2

extern int debug;

#define D_BASIC	(1<<0)
#define D_HASH	(1<<1)
#define D_ADD	(1<<2)
#define D_CHECK	(1<<3)
#define D_BITS	(1<<4)
#define D_INCR	(1<<5)

void hash(u_int8_t *, size_t, hzip_block_t *);
void hzip(char *file);
void hunzip(char *file);
void show_hash(u_int8_t *buf, int blocknum, size_t s);
void show_hb(hzip_block_t *);
char *show_base2(u_int8_t);

u_int8_t mask_set_add(u_int8_t, u_int8_t);
int mask_set_check(u_int8_t, u_int8_t);
u_int8_t mask_unset_add(u_int8_t, u_int8_t);
int mask_unset_check(u_int8_t, u_int8_t);
u_int16_t count_bits(u_int8_t *, size_t);
